from dolphie.Modules.Functions import format_number
from dolphie.Modules.TabManager import Tab
from textual.widgets import DataTable


def create_panel(tab: Tab) -> DataTable:
    dolphie = tab.dolphie

    columns = {
        "rule_id": {"name": "Rule ID", "width": 9, "format": None},
        "hits": {"name": "Total Hits", "width": 10, "format": "number"},
        "hits_s": {"name": "Hits/s", "width": 10, "format": "number"},
        "apply": {"name": "Apply", "width": 5, "format": None},
        "flagIN": {"name": "flagIN", "width": 6, "format": None},
        "flagOUT": {"name": "flagOUT", "width": 7, "format": None},
        "destination_hostgroup": {"name": "Dest HG", "width": 8, "format": None},
        "match_pattern": {"name": "Match Pattern", "width": None, "format": None},
        "match_digest": {"name": "Match Digest", "width": None, "format": None},
    }

    if dolphie.show_additional_query_columns:
        all_columns = [i for i in dolphie.proxysql_mysql_query_rules[0]]
        for column in all_columns:
            if column not in columns:
                columns[column] = {"name": column, "width": None, "format": None}

    mysql_query_rules = tab.proxysql_mysql_query_rules_datatable

    # Clear table if columns change
    if len(mysql_query_rules.columns) != len(columns):
        mysql_query_rules.clear(columns=True)

    # Add columns to the datatable if it is empty
    if not mysql_query_rules.columns:
        for column_key, column_data in columns.items():
            column_name = column_data["name"]
            column_width = column_data["width"]
            mysql_query_rules.add_column(column_name, key=column_key, width=column_width)

    for row in dolphie.proxysql_mysql_query_rules:
        row_id = row["rule_id"]
        row_values = []

        for column_id, (column_key, column_data) in enumerate(columns.items()):
            column_name = column_data["name"]
            column_format = column_data["format"]
            column_value = row.get(column_key)

            # Calculate the values per second for the following columns
            if column_key in ["hits_s"]:
                if not dolphie.proxysql_per_second_data.get(row_id, {}).get(column_key, 0):
                    column_value = 0
                else:
                    value_diff = int(column_value) - dolphie.proxysql_per_second_data.get(row_id, {}).get(column_key, 0)
                    column_value = round(value_diff / dolphie.polling_latency)

                dolphie.proxysql_per_second_data.setdefault(row_id, {})[column_key] = int(row.get(column_key, 0))

            if column_format == "number":
                column_value = format_number(column_value)

            if column_value is None:
                column_value = "[dark_gray]N/A"
            elif column_value == "0":
                column_value = "[dark_gray]0"

            if row_id in mysql_query_rules.rows:
                datatable_value = mysql_query_rules.get_row(row_id)[column_id]

                # Update the datatable if values differ
                if column_value != datatable_value:
                    mysql_query_rules.update_cell(row_id, column_key, column_value)
            else:
                # Create an array of values to append to the datatable
                row_values.append(column_value)

        # Add a new row to the datatable
        if row_values:
            mysql_query_rules.add_row(*row_values, key=row_id)

    # Remove rows from datatable that no longer exist in the data
    if dolphie.proxysql_mysql_query_rules:
        current_rows = {row["rule_id"] for row in dolphie.proxysql_mysql_query_rules}
        existing_rows = set(mysql_query_rules.rows.keys())

        rows_to_remove = existing_rows - current_rows
        for row_id in rows_to_remove:
            mysql_query_rules.remove_row(row_id)
    else:
        if mysql_query_rules.row_count:
            mysql_query_rules.clear()

    tab.proxysql_mysql_query_rules_title.update(
        f"Active Query Rules ([highlight]{mysql_query_rules.row_count}[/highlight])"
    )



from dolphie.Widgets.topbar import TopBar
from rich.style import Style
from textual import events, on
from textual.app import ComposeResult
from textual.containers import Center, Container
from textual.screen import Screen
from textual.widgets import Label, Rule, Static, TextArea
from textual.widgets.text_area import TextAreaTheme


class ProxySQLThreadScreen(Screen):
    AUTO_FOCUS = ""

    CSS = """
        ProxySQLThreadScreen {
            background: #0a0e1b;
        }
        ProxySQLThreadScreen Container {
            height: auto;
        }
        ProxySQLThreadScreen #thread_container {
            margin-top: 1;
            height: auto;
        }
        ProxySQLThreadScreen .title {
            width: 100%;
            content-align: center middle;
            color: #bbc8e8;
            text-style: bold;
        }
        ProxySQLThreadScreen .table {
            content-align: center middle;
            background: #101626;
            border: tall #1d253e;
            padding-left: 1;
            padding-right: 1;
            height: auto;
            width: auto;
        }
        ProxySQLThreadScreen TextArea {
            border: tall #1d253e;
            width: 100;
            height: 35;
        }
    """

    def __init__(
        self,
        connection_status: str,
        app_version: str,
        host: str,
        thread_table: str,
        query: str,
        extended_info: str,
    ):
        super().__init__()

        self.connection_status = connection_status
        self.app_version = app_version
        self.host = host

        self.thread_table = thread_table
        self.formatted_query = query
        self.extended_info = extended_info

        dracula = TextAreaTheme.get_builtin_theme("dracula")
        dracula.base_style = Style(bgcolor="#101626")
        dracula.gutter_style = Style(color="#606e88")
        dracula.cursor_line_gutter_style = Style(color="#95a7c7", bgcolor="#20243b")
        dracula.cursor_line_style = Style(bgcolor="#20243b")
        dracula.selection_style = Style(bgcolor="#293c71")
        dracula.cursor_style = Style(bgcolor="#7a8ab2", color="#121e3a")
        dracula.syntax_styles = {
            "json.label": Style(color="#879bca", bold=True),
            "number": Style(color="#ca87a5"),
        }

        self.extended_info_text_area = TextArea(
            language="json", theme="dracula", show_line_numbers=True, read_only=True
        )
        if self.extended_info:
            self.extended_info_text_area.text = extended_info

    def on_mount(self):
        self.query_one("#thread_table").update(self.thread_table)

        if self.formatted_query:
            self.query_one("#query").update(self.formatted_query)
        else:
            self.query_one("#query_container").display = False

        if not self.extended_info:
            self.query_one("#extended_info_container").display = False

    def compose(self) -> ComposeResult:
        yield TopBar(connection_status=self.connection_status, app_version=self.app_version, host=self.host)

        with Container(id="thread_container"):
            yield Label("Process Details", classes="title")
            yield Center(Static(id="thread_table", shrink=True, classes="table"))

        with Container(id="query_container"):
            yield Rule(line_style="heavy")
            yield Label("Query Details", classes="title")
            yield Center(Static(id="query", shrink=True, classes="table"))

        with Container(id="extended_info_container"):
            yield Rule(line_style="heavy")
            yield Label("Extended Information", classes="title")
            yield Center(self.extended_info_text_area)

    @on(events.Key)
    def on_keypress(self, event: events.Key):
        if event.key == "q":
            if self.screen.is_attached:
                self.app.pop_screen()



from dolphie.Modules.Functions import format_bytes, format_number
from dolphie.Modules.TabManager import Tab
from textual.widgets import DataTable


def create_panel(tab: Tab) -> DataTable:
    dolphie = tab.dolphie

    columns = {
        "hostgroup": {"name": "Hostgroup", "width": None, "format": None},
        "srv_host": {"name": "Backend Host", "width": 35, "format": None},
        "srv_port": {"name": "Port", "width": 5, "format": None},
        "status": {"name": "Status", "width": None, "format": None},
        "weight": {"name": "Weight", "width": None, "format": None},
        "use_ssl": {"name": "SSL", "width": 5, "format": None},
        "ConnUsed": {"name": "Conn Used", "width": 11, "format": "number"},
        "ConnFree": {"name": "Conn Free", "width": 10, "format": "number"},
        "ConnOK": {"name": "Conn OK", "width": 10, "format": "number"},
        "ConnERR": {"name": "Conn ERR", "width": 10, "format": "number"},
        "MaxConnUsed": {"name": "Max Conn", "width": 11, "format": "number"},
        "Queries": {"name": "Queries/s", "width": 10, "format": "number"},
        "Bytes_data_sent": {"name": "Data Sent/s", "width": 11, "format": "bytes"},
        "Bytes_data_recv": {"name": "Data Recvd/s", "width": 12, "format": "bytes"},
        "Latency_us": {"name": "Latency (ms)", "width": 12, "format": "time"},
    }

    hostgroup_summary_datatable = tab.proxysql_hostgroup_summary_datatable

    # Add columns to the datatable if it is empty
    if not hostgroup_summary_datatable.columns:
        for column_key, column_data in columns.items():
            column_name = column_data["name"]
            column_width = column_data["width"]
            hostgroup_summary_datatable.add_column(column_name, key=column_key, width=column_width)

    for row in dolphie.proxysql_hostgroup_summary:
        row_id = f"{row['hostgroup']}_{row['srv_host']}_{row['srv_port']}"
        row_values = []

        for column_id, (column_key, column_data) in enumerate(columns.items()):
            column_name = column_data["name"]
            column_format = column_data["format"]
            column_value = row.get(column_key)

            # Calculate the values per second for the following columns since the total isn't a great metric
            if column_key in ["Queries", "Bytes_data_sent", "Bytes_data_recv"]:
                if not dolphie.proxysql_per_second_data.get(row_id, {}).get(column_key, 0):
                    column_value = 0
                else:
                    value_diff = int(column_value) - dolphie.proxysql_per_second_data.get(row_id, {}).get(column_key, 0)
                    column_value = round(value_diff / dolphie.polling_latency)

                dolphie.proxysql_per_second_data.setdefault(row_id, {})[column_key] = int(row[column_key])
            if column_format == "time":
                column_value = f"{round(int(column_value) / 1000, 2)}"
            elif column_format == "bytes":
                column_value = format_bytes(column_value)
            elif column_format == "number":
                column_value = format_number(column_value)
            elif column_key == "srv_host":
                column_value = dolphie.get_hostname(column_value)
            elif column_key == "status":
                column_value = "[green]ONLINE" if column_value == "ONLINE" else f"[red]{column_value}"
            elif column_key == "use_ssl":
                column_value = "ON" if column_value == "1" else "OFF"

            if column_value == "0" or column_value == 0:
                column_value = "[dark_gray]0"

            if row_id in hostgroup_summary_datatable.rows:
                datatable_value = hostgroup_summary_datatable.get_row(row_id)[column_id]

                # Update the datatable if values differ
                if column_value != datatable_value:
                    hostgroup_summary_datatable.update_cell(row_id, column_key, column_value)
            else:
                # Create an array of values to append to the datatable
                row_values.append(column_value)

        # Add a new row to the datatable
        if row_values:
            hostgroup_summary_datatable.add_row(*row_values, key=row_id)

    # Remove rows from datatable that no longer exist in the data
    if dolphie.proxysql_hostgroup_summary:
        current_rows = {
            f"{row['hostgroup']}_{row['srv_host']}_{row['srv_port']}" for row in dolphie.proxysql_hostgroup_summary
        }
        existing_rows = set(hostgroup_summary_datatable.rows.keys())

        rows_to_remove = existing_rows - current_rows
        for row_id in rows_to_remove:
            hostgroup_summary_datatable.remove_row(row_id)
    else:
        if hostgroup_summary_datatable.row_count:
            hostgroup_summary_datatable.clear()

    tab.proxysql_hostgroup_summary_title.update(
        f"Hostgroups ([highlight]{hostgroup_summary_datatable.row_count}[/highlight])"
    )
